@@ -22,7 +22,7 @@ def parse_ref(current_ref):
         The github reference string.
     """
     if not current_ref.startswith("refs/tags/"):
-        raise Exception(f"Invalid ref `{current_ref}`!")
+        raise Exception(f"Invalid ref `{current_ref}`!")  # noqa
 
     tag_name = current_ref.replace("refs/tags/", "")
     print(tag_name)
