@@ -1,3 +1,4 @@
+"""Notebook format validators."""
 # Copyright (c) IPython Development Team.
 # Distributed under the terms of the Modified BSD License.
 
@@ -24,10 +25,9 @@ def _relax_additional_properties(obj):
     """relax any `additionalProperties`"""
     if isinstance(obj, dict):
         for key, value in obj.items():
-            if key == "additionalProperties":
-                value = True
-            else:
-                value = _relax_additional_properties(value)
+            value = (  # noqa
+                True if key == "additionalProperties" else _relax_additional_properties(value)
+            )
             obj[key] = value
     elif isinstance(obj, list):
         for i, value in enumerate(obj):
@@ -53,10 +53,7 @@ def get_validator(version=None, version_minor=None, relax_add_props=False, name=
     if version_minor is None:
         version_minor = current_minor
 
-    if name:
-        current_validator = _validator_for_name(name)
-    else:
-        current_validator = get_current_validator()
+    current_validator = _validator_for_name(name) if name else get_current_validator()
 
     version_tuple = (current_validator.name, version, version_minor)
 
@@ -100,7 +97,8 @@ def _get_schema_json(v, version=None, version_minor=None):
         # load the latest schema
         schema_path = os.path.join(os.path.dirname(v.__file__), v.nbformat_schema[(None, None)])
     else:
-        raise AttributeError("Cannot find appropriate nbformat schema file.")
+        msg = "Cannot find appropriate nbformat schema file."
+        raise AttributeError(msg)
     with open(schema_path) as f:
         schema_json = json.load(f)
     return schema_json
@@ -125,7 +123,8 @@ def isvalid(nbjson, ref=None, version=None, version_minor=None):
     else:
         return True
     finally:
-        assert nbjson == orig
+        if nbjson != orig:
+            raise AssertionError
 
 
 def _format_as_index(indices):
@@ -182,11 +181,13 @@ class NotebookValidationError(ValidationError):
     """
 
     def __init__(self, original, ref=None):
+        """Initialize the error class."""
         self.original = original
         self.ref = getattr(self.original, "ref", ref)
         self.message = self.original.message
 
     def __getattr__(self, key):
+        """Get an attribute from the error."""
         return getattr(self.original, key)
 
     def __unicode__(self):
@@ -223,10 +224,11 @@ def better_validation_error(error, version, version_minor):
     if it's a cell type or output_type error,
     try validating directly based on the type for a better error message
     """
+    if not len(error.schema_path):
+        return error
     key = error.schema_path[-1]
     ref = None
     if key.endswith("Of"):
-
         if isinstance(error.instance, dict):
             if "cell_type" in error.instance:
                 ref = error.instance["cell_type"] + "_cell"
@@ -249,7 +251,7 @@ def better_validation_error(error, version, version_minor):
                 if better.ref is None:
                     better.ref = ref
                 return better
-            except Exception:
+            except Exception:  # noqa
                 # if it fails for some reason,
                 # let the original error through
                 pass
@@ -262,7 +264,8 @@ def normalize(
     version_minor: Optional[int] = None,
     *,
     relax_add_props: bool = False,
-) -> Tuple[Any, int]:
+    strip_invalid_metadata: bool = False,
+) -> Tuple[int, Any]:
     """
     Normalise a notebook prior to validation.
 
@@ -280,15 +283,18 @@ def normalize(
     version : int
     version_minor : int
     relax_add_props : bool
-        Wether to allow extra property in the Json schema validating the
+        Whether to allow extra property in the Json schema validating the
         notebook.
+    strip_invalid_metadata : bool
+        Whether to strip metadata that does not exist in the Json schema when
+        validating the notebook.
 
     Returns
     -------
-    notebook : dict
-        deep-copy of the original object with relevant changes.
     changes : int
         number of changes in the notebooks
+    notebook : dict
+        deep-copy of the original object with relevant changes.
 
     """
     nbdict = deepcopy(nbdict)
@@ -303,7 +309,7 @@ def normalize(
         version_minor,
         True,
         relax_add_props=relax_add_props,
-        strip_invalid_metadata=False,
+        strip_invalid_metadata=strip_invalid_metadata,
     )
 
 
@@ -314,7 +320,7 @@ def _normalize(
     repair_duplicate_cell_ids: bool,
     relax_add_props: bool,
     strip_invalid_metadata: bool,
-) -> Tuple[Any, int]:
+) -> Tuple[int, Any]:
     """
     Private normalisation routine.
 
@@ -366,7 +372,8 @@ def _normalize(
                         stacklevel=3,
                     )
                 else:
-                    raise ValidationError(f"Non-unique cell id '{cell_id}' detected.")
+                    msg = f"Non-unique cell id '{cell_id}' detected."
+                    raise ValidationError(msg)
             seen_ids.add(cell_id)
     if strip_invalid_metadata:
         changes += _strip_invalida_metadata(
@@ -381,7 +388,7 @@ def _dep_warn(field):
             f"""`{field}` kwargs of validate has been deprecated for security
         reasons, and will be removed soon.
 
-        Please explicitly use the `new_notebook,n_changes = nbformat.validator.normalize(old_notebook, ...)` if you wish to
+        Please explicitly use the `n_changes, new_notebook = nbformat.validator.normalize(old_notebook, ...)` if you wish to
         normalise your notebook. `normalize` is available since nbformat 5.5.0
 
         """
@@ -391,17 +398,16 @@ def _dep_warn(field):
     )
 
 
-def validate(
+def validate(  # noqa
     nbdict: Any = None,
     ref: Optional[str] = None,
     version: Optional[int] = None,
     version_minor: Optional[int] = None,
-    relax_add_props: bool = _deprecated,  # type: ignore
+    relax_add_props: bool = False,
     nbjson: Any = None,
     repair_duplicate_cell_ids: bool = _deprecated,  # type: ignore
     strip_invalid_metadata: bool = _deprecated,  # type: ignore
 ) -> None:
-
     """Checks whether the given notebook dict-like object
     conforms to the relevant notebook format schema.
 
@@ -415,14 +421,13 @@ def validate(
     version : int
     version_minor : int
     relax_add_props : bool
-        Deprecated since 5.5.0 – will be removed in the future.
-        Wether to allow extra property in the Json schema validating the
-        notebook.
+        Wether to allow extra properties in the JSON schema validating the notebook.
+        When True, all known fields are validated, but unknown fields are ignored.
     nbjson
-    repair_duplicate_cell_ids : boolny
-        Deprecated since 5.5.0 – will be removed in the future.
+    repair_duplicate_cell_ids : bool
+        Deprecated since 5.5.0 - will be removed in the future.
     strip_invalid_metadata : bool
-        Deprecated since 5.5.0 – will be removed in the future.
+        Deprecated since 5.5.0 - will be removed in the future.
 
     Returns
     -------
@@ -440,12 +445,7 @@ def validate(
 
     Please explicitly call `normalize` if you need to normalize notebooks.
     """
-    assert isinstance(ref, str) or ref is None
-
-    if relax_add_props is _deprecated:
-        relax_add_props = False
-    else:
-        _dep_warn("relax_add_props")
+    assert isinstance(ref, str) or ref is None  # noqa
 
     if strip_invalid_metadata is _deprecated:
         strip_invalid_metadata = False
@@ -465,7 +465,8 @@ def validate(
     elif nbjson is not None:
         nbdict = nbjson
     else:
-        raise TypeError("validate() missing 1 required argument: 'nbdict'")
+        msg = "validate() missing 1 required argument: 'nbdict'"
+        raise TypeError(msg)
 
     if ref is None:
         # if ref is not specified, we have a whole notebook, so we can get the version
@@ -476,12 +477,12 @@ def validate(
             version_minor = nbdict_version_minor
     else:
         # if ref is specified, and we don't have a version number, assume we're validating against 1.0
-        if version is None:
+        if version is None:  # noqa
             version, version_minor = 1, 0
 
     if ref is None:
-        assert isinstance(version, int)
-        assert isinstance(version_minor, int)
+        assert isinstance(version, int)  # noqa
+        assert isinstance(version_minor, int)  # noqa
         _normalize(
             nbdict,
             version,
@@ -499,10 +500,30 @@ def validate(
         relax_add_props=relax_add_props,
         strip_invalid_metadata=strip_invalid_metadata,
     ):
-
         raise error
 
 
+def _get_errors(
+    nbdict: Any, version: int, version_minor: int, relax_add_props: bool, *args: Any
+) -> Any:
+    validator = get_validator(version, version_minor, relax_add_props=relax_add_props)
+    if not validator:
+        msg = f"No schema for validating v{version}.{version_minor} notebooks"
+        raise ValidationError(msg)
+    iter_errors = validator.iter_errors(nbdict, *args)
+    errors = list(iter_errors)
+    # jsonschema gives the best error messages.
+    if len(errors) and validator.name != "jsonschema":
+        validator = get_validator(
+            version=version,
+            version_minor=version_minor,
+            relax_add_props=relax_add_props,
+            name="jsonschema",
+        )
+        return validator.iter_errors(nbdict, *args)
+    return iter(errors)
+
+
 def _strip_invalida_metadata(
     nbdict: Any, version: int, version_minor: int, relax_add_props: bool
 ) -> int:
@@ -527,22 +548,20 @@ def _strip_invalida_metadata(
         number of modifications
 
     """
-    validator = get_validator(version, version_minor, relax_add_props=relax_add_props)
-    if not validator:
-        raise ValidationError(f"No schema for validating v{version}.{version_minor} notebooks")
-    errors = [e for e in validator.iter_errors(nbdict)]
-
+    errors = _get_errors(nbdict, version, version_minor, relax_add_props)
     changes = 0
-    if len(errors) > 0:
-        if validator.name == "fastjsonschema":
-            validator = get_validator(
-                version=version,
-                version_minor=version_minor,
-                relax_add_props=relax_add_props,
-                name="jsonschema",
-            )
-            errors = list(validator.iter_errors(nbdict))
-
+    if len(list(errors)) > 0:
+        # jsonschema gives a better error tree.
+        validator = get_validator(
+            version=version,
+            version_minor=version_minor,
+            relax_add_props=relax_add_props,
+            name="jsonschema",
+        )
+        if not validator:
+            msg = f"No jsonschema for validating v{version}.{version_minor} notebooks"
+            raise ValidationError(msg)
+        errors = validator.iter_errors(nbdict)
         error_tree = validator.error_tree(errors)
         if "metadata" in error_tree:
             for key in error_tree["metadata"]:
@@ -569,7 +588,7 @@ def _strip_invalida_metadata(
                             rel_path = error.relative_path
                             error_for_intended_schema = error.schema_path[0] == schema_index
                             is_top_level_metadata_key = (
-                                len(rel_path) == 2 and rel_path[0] == "metadata"
+                                len(rel_path) == 2 and rel_path[0] == "metadata"  # noqa
                             )
                             if error_for_intended_schema and is_top_level_metadata_key:
                                 nbdict["cells"][cell_idx]["metadata"].pop(rel_path[1], None)
@@ -604,20 +623,25 @@ def iter_validate(
     elif nbjson is not None:
         nbdict = nbjson
     else:
-        raise TypeError("iter_validate() missing 1 required argument: 'nbdict'")
+        msg = "iter_validate() missing 1 required argument: 'nbdict'"
+        raise TypeError(msg)
 
     if version is None:
         version, version_minor = get_version(nbdict)
 
-    validator = get_validator(version, version_minor, relax_add_props=relax_add_props)
-
-    if validator is None:
-        # no validator
-        yield ValidationError("No schema for validating v%s notebooks" % version)
-        return
-
     if ref:
-        errors = validator.iter_errors(nbdict, {"$ref": "#/definitions/%s" % ref})
+        try:
+            errors = _get_errors(
+                nbdict,
+                version,
+                version_minor,
+                relax_add_props,
+                {"$ref": "#/definitions/%s" % ref},
+            )
+        except ValidationError as e:
+            yield e
+            return
+
     else:
         if strip_invalid_metadata:
             _strip_invalida_metadata(nbdict, version, version_minor, relax_add_props)
@@ -626,7 +650,11 @@ def iter_validate(
         # didn't cause another complex validation issue in the schema.
         # Also to ensure that higher-level errors produced by individual metadata validation
         # failures are removed.
-        errors = validator.iter_errors(nbdict)
+        try:
+            errors = _get_errors(nbdict, version, version_minor, relax_add_props)
+        except ValidationError as e:
+            yield e
+            return
 
     for error in errors:
         yield better_validation_error(error, version, version_minor)
